@@ -2,7 +2,7 @@
     This file is part of Akonadi Contact.
 
     Copyright (c) 2009 - 2010 Tobias Koenig <tokoe@kde.org>
-    SPDX-FileCopyrightText: 2010 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+    SPDX-FileCopyrightText: 2010 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>
     SPDX-FileCopyrightText: 2010 Andras Mantia <andras@kdab.com>
 
     SPDX-License-Identifier: LGPL-2.0-or-later
@@ -46,13 +46,19 @@ public:
     Private(KActionCollection *actionCollection, QWidget *parentWidget, StandardMailActionManager *parent)
         : mActionCollection(actionCollection)
         , mParentWidget(parentWidget)
-        , mCollectionSelectionModel(nullptr)
-        , mItemSelectionModel(nullptr)
         , mParent(parent)
     {
         mGenericManager = new StandardActionManager(actionCollection, parentWidget);
 
-        mParent->connect(mGenericManager, &StandardActionManager::actionStateUpdated,
+        QObject::connect(mGenericManager, &StandardActionManager::selectionsChanged,
+                         mParent, [this](const Collection::List &selectedCollections,
+                                         const Collection::List &selectedFavoriteCollections,
+                                         const Item::List &selectedItems) {
+                             // Optimization: pass along the lists to avoid recalculating them here
+                             updateActions(selectedCollections, selectedFavoriteCollections, selectedItems);
+                         });
+
+        QObject::connect(mGenericManager, &StandardActionManager::actionStateUpdated,
                          mParent, &StandardMailActionManager::actionStateUpdated);
 
         mGenericManager->setMimeTypeFilter(QStringList() << KMime::Message::mimeType());
@@ -313,10 +319,12 @@ public:
     {
         const Akonadi::Item::List selectedItems = mGenericManager->selectedItems();
         const Akonadi::Collection::List selectedCollections = mGenericManager->selectedCollections();
+        updateActions(selectedCollections, {}, selectedItems);
+    }
 
-        bool itemIsSelected = !selectedItems.isEmpty();
-        bool collectionIsSelected = !selectedCollections.isEmpty();
-
+    void updateActionsFromSelectedItems(const Item::List &selectedItems)
+    {
+        const bool itemIsSelected = !selectedItems.isEmpty();
         if (itemIsSelected) {
             bool allMarkedAsImportant = true;
             bool allMarkedAsRead = true;
@@ -392,6 +400,16 @@ public:
                 action->setEnabled(false);
             }
         }
+    }
+
+    void updateActions(const Collection::List &selectedCollections,
+                       const Collection::List &selectedFavoriteCollections,
+                       const Item::List &selectedItems)
+    {
+        Q_UNUSED(selectedFavoriteCollections);
+        const bool itemIsSelected = !selectedItems.isEmpty();
+        const bool collectionIsSelected = !selectedCollections.isEmpty();
+        updateActionsFromSelectedItems(selectedItems);
 
         bool enableMarkAllAsRead = false;
         bool enableMarkAllAsUnread = false;
@@ -470,17 +488,8 @@ public:
         action->setData(data);
     }
 
-    void slotMarkAs()
+    void markItemsAs(QByteArray typeStr, const Akonadi::Item::List &items, bool checkIntercept = true)
     {
-        const QAction *action = qobject_cast<QAction *>(mParent->sender());
-        Q_ASSERT(action);
-
-        const Akonadi::Item::List items = mGenericManager->selectedItems();
-        if (items.isEmpty()) {
-            return;
-        }
-
-        QByteArray typeStr = action->data().toByteArray();
         qCDebug(AKONADIMIME_LOG) << "Mark mail as: " << typeStr;
 
         bool invert = false;
@@ -503,26 +512,33 @@ public:
             type = MarkMailAsImportant;
         }
 
-        if (mInterceptedActions.contains(type)) {
+        if (mInterceptedActions.contains(type) && checkIntercept) {
             return;
         }
 
-        MarkAsCommand *command = new MarkAsCommand(targetStatus, items, invert, mParent);
+        auto *command = new MarkAsCommand(targetStatus, items, invert, mParent);
         command->execute();
     }
 
-    void slotMarkAllAs()
+    void slotMarkAs()
     {
+        const Akonadi::Item::List items = mGenericManager->selectedItems();
+        if (items.isEmpty()) {
+            return;
+        }
         const QAction *action = qobject_cast<QAction *>(mParent->sender());
         Q_ASSERT(action);
 
-        QByteArray typeStr = action->data().toByteArray();
+
+        const QByteArray typeStr = action->data().toByteArray();
+
+        markItemsAs(typeStr, items, true);
+    }
+
+    void markAllItemsAs(QByteArray typeStr, const Akonadi::Collection::List &collections, bool checkIntercept = true)
+    {
         qCDebug(AKONADIMIME_LOG) << "Mark all as: " << typeStr;
 
-        const Akonadi::Collection::List collections = mGenericManager->selectedCollections();
-        if (collections.isEmpty()) {
-            return;
-        }
 
         Akonadi::MessageStatus targetStatus;
         targetStatus.setStatusFromStr(QLatin1String(typeStr));
@@ -549,14 +565,27 @@ public:
             type = MarkAllMailAsImportant;
         }
 
-        if (mInterceptedActions.contains(type)) {
+        if (mInterceptedActions.contains(type) && checkIntercept) {
             return;
         }
 
-        MarkAsCommand *command = new MarkAsCommand(targetStatus, collections, invert, recursive, mParent);
+        auto *command = new MarkAsCommand(targetStatus, collections, invert, recursive, mParent);
         command->execute();
     }
 
+    void slotMarkAllAs()
+    {
+        const Akonadi::Collection::List collections = mGenericManager->selectedCollections();
+        if (collections.isEmpty()) {
+            return;
+        }
+
+        const QAction *action = qobject_cast<QAction *>(mParent->sender());
+        Q_ASSERT(action);
+        const QByteArray typeStr = action->data().toByteArray();
+        markAllItemsAs(typeStr, collections, true);
+    }
+
     void slotMoveToTrash()
     {
         if (mInterceptedActions.contains(StandardMailActionManager::MoveToTrash)) {
@@ -572,7 +601,7 @@ public:
             return;
         }
 
-        MoveToTrashCommand *command = new MoveToTrashCommand(mCollectionSelectionModel->model(), items, mParent);
+        auto *command = new MoveToTrashCommand(mCollectionSelectionModel->model(), items, mParent);
         command->execute();
     }
 
@@ -591,7 +620,7 @@ public:
             return;
         }
 
-        MoveToTrashCommand *command = new MoveToTrashCommand(mCollectionSelectionModel->model(), collections, mParent);
+        auto *command = new MoveToTrashCommand(mCollectionSelectionModel->model(), collections, mParent);
         command->execute();
     }
 
@@ -606,7 +635,7 @@ public:
             return;
         }
 
-        RemoveDuplicatesJob *job = new RemoveDuplicatesJob(collections, mParent);
+        auto *job = new RemoveDuplicatesJob(collections, mParent);
         connect(job, &RemoveDuplicatesJob::finished, mParent, [this](KJob *job) {
             slotJobFinished(job);
         });
@@ -625,7 +654,7 @@ public:
             return;
         }
 
-        EmptyTrashCommand *command = new EmptyTrashCommand(const_cast<QAbstractItemModel *>(mCollectionSelectionModel->model()), mParent);
+        auto *command = new EmptyTrashCommand(const_cast<QAbstractItemModel *>(mCollectionSelectionModel->model()), mParent);
         command->execute();
     }
 
@@ -644,18 +673,18 @@ public:
             return;
         }
 
-        EmptyTrashCommand *command = new EmptyTrashCommand(collections.first(), mParent);
+        auto *command = new EmptyTrashCommand(collections.first(), mParent);
         command->execute();
     }
 
-    KActionCollection *mActionCollection = nullptr;
-    QWidget *mParentWidget = nullptr;
+    KActionCollection *const mActionCollection;
+    QWidget *const mParentWidget;
     StandardActionManager *mGenericManager = nullptr;
     QItemSelectionModel *mCollectionSelectionModel = nullptr;
     QItemSelectionModel *mItemSelectionModel = nullptr;
     QHash<StandardMailActionManager::Type, QAction *> mActions;
     QSet<StandardMailActionManager::Type> mInterceptedActions;
-    StandardMailActionManager *mParent = nullptr;
+    StandardMailActionManager *const mParent;
 };
 
 StandardMailActionManager::StandardMailActionManager(KActionCollection *actionCollection, QWidget *parent)
@@ -680,21 +709,22 @@ void StandardMailActionManager::setCollectionSelectionModel(QItemSelectionModel
     connect(selectionModel->model(), &QAbstractItemModel::rowsRemoved, this, [this]() {
         d->updateActions();
     });
-    connect(selectionModel, &QItemSelectionModel::selectionChanged, this, [this]() {
-        d->updateActions();
-    });
+    // connecting to QItemSelectionModel::selectionChanged is done by mGenericManager
 
     d->updateActions();
 }
 
+void StandardMailActionManager::setItems(const Item::List &selectedItems)
+{
+    d->updateActionsFromSelectedItems(selectedItems);
+}
+
 void StandardMailActionManager::setItemSelectionModel(QItemSelectionModel *selectionModel)
 {
     d->mItemSelectionModel = selectionModel;
     d->mGenericManager->setItemSelectionModel(selectionModel);
 
-    connect(selectionModel, &QItemSelectionModel::selectionChanged, this, [this]() {
-        d->updateActions();
-    });
+    // connecting to QItemSelectionModel::selectionChanged is done by mGenericManager
 
     //to catch item modifications, listen to the model's dataChanged signal as well
     connect(selectionModel->model(), &QAbstractItemModel::dataChanged, this, [this]() {
@@ -1000,4 +1030,14 @@ Akonadi::StandardActionManager *StandardMailActionManager::standardActionManager
     return d->mGenericManager;
 }
 
+void StandardMailActionManager::markItemsAs(const QByteArray &typeStr, const Item::List &items, bool checkIntercept)
+{
+    d->markItemsAs(typeStr, items, checkIntercept);
+}
+
+void StandardMailActionManager::markAllItemsAs(const QByteArray &typeStr, const Collection::List &collections, bool checkIntercept)
+{
+    d->markAllItemsAs(typeStr, collections, checkIntercept);
+}
+
 #include "moc_standardmailactionmanager.cpp"
